import { world, system, MinecraftDimensionTypes, BlockPermutation } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";

let teleporterLocations = {
    overworld: {
        white: [],
        black: [],
        red: [],
        blue: [],
        green: [],
        yellow: [],
        orange: [],
        purple: [],
        cyan: [],
        brown: [],
        pink: [],
        gray: [],
        light_blue: [],
        silver: [],
        lime: [],
        magenta: []
    },
    nether: {
        white: [],
        black: [],
        red: [],
        blue: [],
        green: [],
        yellow: [],
        orange: [],
        purple: [],
        cyan: [],
        brown: [],
        pink: [],
        gray: [],
        light_blue: [],
        silver: [],
        lime: [],
        magenta: []
    },
    end: {
        white: [],
        black: [],
        red: [],
        blue: [],
        green: [],
        yellow: [],
        orange: [],
        purple: [],
        cyan: [],
        brown: [],
        pink: [],
        gray: [],
        light_blue: [],
        silver: [],
        lime: [],
        magenta: []
    }
};

function getDimensionName(dimensionId) {
    switch(dimensionId) {
        case MinecraftDimensionTypes.overworld:
            return "overworld";
        case MinecraftDimensionTypes.nether:
            return "nether";
        case MinecraftDimensionTypes.theEnd:
            return "end";
        default:
            return null;
    }
}

function loadTeleporterData() {
    const data = world.getDynamicProperty("teleporterLocations");
    return data ? JSON.parse(data) : {
        overworld: {},
        nether: {},
        end: {}
    };
}

function saveTeleporterData(data) {
    world.setDynamicProperty("teleporterLocations", JSON.stringify(data));
}

function addTeleporter(dimension, type, location) {
    const teleporterData = loadTeleporterData();
    const dimensionName = getDimensionName(dimension);
    if (dimensionName && !teleporterData[dimensionName][type]) {
        teleporterData[dimensionName][type] = [];
    }
    teleporterData[dimensionName][type].push(location);
    saveTeleporterData(teleporterData);
}

function removeTeleporter(dimension, type, location) {
    const teleporterData = loadTeleporterData();
    const dimensionName = getDimensionName(dimension);
    if (dimensionName && teleporterData[dimensionName][type]) {
        teleporterData[dimensionName][type] = teleporterData[dimensionName][type].filter(loc => 
            loc.x !== location.x || loc.y !== location.y || loc.z !== location.z
        );
        saveTeleporterData(teleporterData);
    }
}

function showTeleporterSelectionUI(player, dimension, type) {
    const form = new ActionFormData()
        .title(`${type} teleporter`)
        .body("Please select a teleport destination");

    const teleporterData = loadTeleporterData();

    for (const [dimensionName, types] of Object.entries(teleporterData)) {
        if (types[type]) {
            types[type].forEach(location => {
                form.button(`(${location.x}, ${location.y}, ${location.z}) in ${dimensionName}`);
            });
        }
    }

    form.show(player).then(response => {
        if (response.canceled) {
            return;
        }

        let selectedLocation = null;
        let selectedDimension = null;
        let buttonIndex = response.selection;
        
        for (const [dimensionName, types] of Object.entries(teleporterData)) {
            if (types[type]) {
                if (buttonIndex < types[type].length) {
                    selectedLocation = types[type][buttonIndex];
                    selectedDimension = dimensionName;
                    break;
                } else {
                    buttonIndex -= types[type].length;
                }
            }
        }

        if (selectedLocation && selectedDimension) {
            const dimId = selectedDimension === "overworld" ? "overworld" :
                          selectedDimension === "nether" ? "nether" : "the_end";
            
            try {
                player.runCommandAsync("gamerule sendcommandfeedback false");
                player.runCommandAsync(`execute in ${dimId} run tp @s ${selectedLocation.x + 0.5} ${selectedLocation.y + 1} ${selectedLocation.z + 0.5}`);
                player.playSound("beacon.power", { volume: 0.7, pitch: 1 });
                player.removeTag(type);
            } catch (error) {
                console.error(`Failed to teleport player: ${error}`);
            }
        }
    }).catch(error => {
        console.error(`Error showing UI: ${error}`);
    });
}

world.afterEvents.playerPlaceBlock.subscribe(event => {
    const block = event.block;
    const blockTypeId = block.typeId;
    const dimension = block.dimension.id;

    if (blockTypeId.startsWith("tunekeshi:teleporter_")) {
        const type = blockTypeId.replace("tunekeshi:teleporter_", "");
        addTeleporter(dimension, type, block.location);
    }
});

world.beforeEvents.playerBreakBlock.subscribe(event => {
    const block = event.block;
    const blockTypeId = block.typeId;
    const dimension = block.dimension.id;

    if (blockTypeId.startsWith("tunekeshi:teleporter_")) {
        const type = blockTypeId.replace("tunekeshi:teleporter_", "");
        removeTeleporter(dimension, type, block.location);
    }
});

function checkAndShowTeleportUI() {
    for (const player of world.getPlayers()) {
        const tags = player.getTags();
        tags.forEach(tag => {
            if (Object.keys(loadTeleporterData().overworld).includes(tag) ||
                Object.keys(loadTeleporterData().nether).includes(tag) ||
                Object.keys(loadTeleporterData().end).includes(tag)) {
                showTeleporterSelectionUI(player, player.dimension.id, tag);
                player.removeTag(tag);
            }
        });
    }
}

system.runInterval(() => {
    checkAndShowTeleportUI();
}, 20);